==> commonc++-1.9.0.tar.gz: CommonC++-1.9.0/src/event.cpp <==
// Copyright (C) 1999-2001 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// 
// As a special exception to the GNU General Public License, permission is 
// granted for additional uses of the text contained in its release 
// of Common C++.
// 
// The exception is that, if you link the Common C++ library with other
// files to produce an executable, this does not by itself cause the
// resulting executable to be covered by the GNU General Public License.
// Your use of that executable is in no way restricted on account of
// linking the Common C++ library code into it.
// 
// This exception does not however invalidate any other reasons why
// the executable file might be covered by the GNU General Public License.
// 
// This exception applies only to the code released under the 
// name Common C++.  If you copy code from other releases into a copy of
// Common C++, as the General Public License permits, the exception does
// not apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
// 
// If you write modifications of your own for Common C++, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice.  

#include "config.h"
#include "export.h"
#include "strchar.h"
#include "exception.h"
#include "thread.h"
#include <cerrno>

#ifdef	__NAMESPACES__
namespace ost {
#endif

Event::Event() : 
Mutex()
{
	pthread_cond_init(&_cond, NULL);
	_signaled = false;
	_count = 0;
}

void Event::Signal(void)
{
	ENTER_CRITICAL
	_signaled = true;
	++_count;
	pthread_cond_broadcast(&_cond);
	LEAVE_CRITICAL		
}

bool Event::Wait(timeout_t timer)
{
	int	rc = 0;
	struct	timespec spec;

	ENTER_CRITICAL
	long count = _count;
	while(!_signaled && _count == count)
	{
		if(timer != TIMEOUT_INF)
			rc = pthread_cond_timedwait(
				&_cond, &_mutex, gettimeout(&spec, timer));
		else
			pthread_cond_wait(&_cond, &_mutex);
		if(rc == ETIMEDOUT)
			break;
	}
	LEAVE_CRITICAL
	if(rc == ETIMEDOUT)
		return false;
	return true;
}

#ifdef	__NAMESPACES__
};
#endif
	
/** EMACS **
 * Local variables:
 * mode: c++
 * c-basic-offset: 8
 * End:
 */

==> commonc++-1.9.1.tar.gz: CommonC++-1.9.1/src/event.cpp <==
// Copyright (C) 1999-2001 Open Source Telecom Corporation.
//  
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software 
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
// 
// As a special exception to the GNU General Public License, permission is 
// granted for additional uses of the text contained in its release 
// of Common C++.
// 
// The exception is that, if you link the Common C++ library with other
// files to produce an executable, this does not by itself cause the
// resulting executable to be covered by the GNU General Public License.
// Your use of that executable is in no way restricted on account of
// linking the Common C++ library code into it.
// 
// This exception does not however invalidate any other reasons why
// the executable file might be covered by the GNU General Public License.
// 
// This exception applies only to the code released under the 
// name Common C++.  If you copy code from other releases into a copy of
// Common C++, as the General Public License permits, the exception does
// not apply to the code that you add in this way.  To avoid misleading
// anyone as to the status of such modified files, you must delete
// this exception notice from them.
// 
// If you write modifications of your own for Common C++, it is your choice
// whether to permit this exception to apply to your modifications.
// If you do not wish that, delete this exception notice.  

#include "config.h"
#include "export.h"
#include "strchar.h"
#include "exception.h"
#include "thread.h"
#include <cerrno>

#ifdef	__NAMESPACES__
namespace ost {
#endif

Event::Event() 
{
	pthread_cond_init(&_cond, NULL);
	_signaled = false;
	_count = 0;
}

Event::~Event()
{
	pthread_cond_destroy(&_cond);
}

void Event::Signal(void)
{
	mutex.EnterMutex();
	_signaled = true;
	++_count;
	pthread_cond_broadcast(&_cond);
	mutex.LeaveMutex();
}

void Event::Reset(void)
{
	_signaled = false;
}

bool Event::Wait(void)
{
	return Wait(0);
}

bool Event::Wait(timeout_t timer)
{
	int	rc = 0;
	struct	timespec spec;

	mutex.EnterMutex();
	long count = _count;
	while(!_signaled && _count == count)
	{
		if(timer != TIMEOUT_INF)
			rc = pthread_cond_timedwait(
				&_cond, &mutex._mutex, gettimeout(&spec, timer));
		else
			pthread_cond_wait(&_cond, &mutex._mutex);
		if(rc == ETIMEDOUT)
			break;
	}
	mutex.LeaveMutex();
	if(rc == ETIMEDOUT)
		return false;
	return true;
}

#ifdef	__NAMESPACES__
};
#endif
	
/** EMACS **
 * Local variables:
 * mode: c++
 * c-basic-offset: 8
 * End:
 */
